package com.persagy.apm.report.paragraphs.service.impl;

import com.persagy.apm.report.userconfig.paragraph.model.Paragraph;
import com.persagy.apm.report.detail.enums.ParagraphContentItemTypeEnum;
import com.persagy.apm.report.detail.model.vo.ItemVO;
import com.persagy.apm.report.detail.paragraphvalue.model.vo.ParagraphContentVO;
import com.persagy.apm.report.outline.model.ReportOutline;
import com.persagy.apm.report.outline.service.IReportOutlineService;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 区域段落（行为指标）建造者实现类
 *
 * @author lixing
 * @version V1.0 2021/7/20 9:42 上午
 **/
public abstract class AreaIndicatorsParagraphBuilder extends ParagraphBuilder {
    @Autowired
    IReportOutlineService reportOutlineService;

    /**
     * 计算段落内容部分
     *
     * @param paragraph     段落对象
     * @param reportOutline 报告概要实体
     * @return 段落内容部分
     * @author lixing
     * @version V1.0 2021/8/2 9:43 上午
     */
    @Override
    public ParagraphContentVO getParagraphContent(Paragraph paragraph, ReportOutline reportOutline) {
        return getIndicatorsParagraphContent(paragraph, reportOutline);
    }

    /**
     * 计算段落内容部分（行为指标）
     *
     * @param paragraph     段落对象
     * @param reportOutline 报告概要实体
     * @return 段落内容部分
     * @author lixing
     * @version V1.0 2021/8/2 10:47 上午
     */
    public ParagraphContentVO getIndicatorsParagraphContent(Paragraph paragraph, ReportOutline reportOutline) {
        /* 获取段落关联的条目 */
        List<String> itemIdList = paragraphService.getRelItemIds(paragraph);
        // 报告关联的项目id列表
        List<String> projectIds = reportOutlineService.queryRelatedProjectIds(reportOutline);
        if (itemIdList == null || projectIds == null) {
            return null;
        }

        ParagraphContentVO paragraphContentVO = new ParagraphContentVO();
        List<ItemVO> items = new ArrayList<>();
        paragraphContentVO.setItems(items);

        for (String itemId : itemIdList) {
            ItemVO itemVO = getIndicatorItem(reportOutline.getReportMonth(), projectIds, itemId);
            items.add(itemVO);
        }
        paragraphContentVO.setItemType(ParagraphContentItemTypeEnum.FUNCTION);
        return paragraphContentVO;
    }

    /**
     * 拼装指标对象
     *
     * @param reportDate 报告月份
     * @param projectIds 要统计的项目id列表
     * @param functionId 指标id
     * @return 指标对象
     * @author lixing
     * @version V1.0 2021/8/5 2:13 下午
     */
    public abstract ItemVO getIndicatorItem(Date reportDate, List<String> projectIds, String functionId);
}
